/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global openDatabase
 */
var DB_VERSION = 1.0,
    DB_NAME = "MoneyBook",
    DB_DISPLAY_NAME = "moneybook_db",
    DB_SIZE = 2 * 1024 * 1024,
    DB_TABLE_NAME = "tizenMoneybook",
    db,
    dataTypeList = ["id", "item", "price", "insertDay"],
    pageList = ["pageMain", "pageInput", "pageResult"],
    pageNow = "pageMain";

function onSuccess(e) {
    console.log("Success : " + e.message);
}

function onError(e) {
    console.warn("Error : " + e.message);
}

function getDateTime() {
    var day = new Date();

    return ((day.getMonth() + 1) + "/" + day.getDate() + " " + day.getHours() + ":" + day.getMinutes());
}

// Open the database
function openDB() {
    if (window.openDatabase) {
        db = openDatabase(DB_NAME, DB_VERSION, DB_DISPLAY_NAME, DB_SIZE, function() {
            onSuccess({
                message: "Database Creation Complete"
            });
        });
    } else {
        onError({
            message: "WebSQL not supported"
        });
    }
}

function setResult(resultElmID, data) {
    var divResult = document.querySelector("#" + resultElmID);

    if (data) {
        divResult.appendChild(document.createTextNode(data.message));
        setTimeout(function() {
            setResult(resultElmID, null);
        }, 3000);
    } else {
        divResult.innerHTML = "";
    }
}

// Create the table if not exists
function createTable(db) {
    db.transaction(function(t) {
        t.executeSql("CREATE TABLE if not exists " + DB_TABLE_NAME + " (id INTEGER PRIMARY KEY, item TEXT, price INTEGER, insertDay DATETIME)", []);
    });
}

// Insert a data to the table
function insertData(db, data) {
    db.transaction(function(t) {
        var dayString;

        dayString = getDateTime();
        t.executeSql("INSERT INTO " + DB_TABLE_NAME + " (item, price, insertDay) VALUES (?, ?, ?)", [data.item, data.price, dayString]);
    });
}

// Delete a data from the table
function deleteData(db, data) {
    db.transaction(function(t) {
        t.executeSql("DELETE FROM " + DB_TABLE_NAME + " WHERE id = ?", [data.id]);
    });
}

// Delete all data from the table
function clearDataView(db) {
    var objResultContents = document.querySelector("#resultDetail");

    objResultContents.innerHTML = "";
    db.transaction(function(t) {
        t.executeSql("DELETE FROM " + DB_TABLE_NAME + " WHERE id > 0", []);
    });
}

function createDeleteCallback(i, objTable, objRow) {
    var retFunc = function() {
        var data = {
            id: i,
            table: objTable,
            row: objRow
        };

        deleteData(db, data);
        data.table.removeChild(data.row);
    };

    return retFunc;
}

function showDataView(dataArray) {
    var objResultContents = document.querySelector("#resultDetail"),
        objTable,
        objRow,
        objCol,
        i,
        prop;

    objResultContents.innerHTML = "";

    objTable = document.createElement("div");
    objTable.className = "resultTable";

    for (i = 0; i < dataArray.length; i++) {
        objRow = document.createElement("div");

        objRow.className = "resultTableRow";
        for (prop in dataArray[i]) {
            if (dataArray[i].hasOwnProperty(prop)) {
                objCol = document.createElement("div");
                if (prop === "id") {
                    objCol.addEventListener("click", createDeleteCallback(dataArray[i][prop], objTable, objRow));
                }
                objCol.className = prop + "Detail";
                objCol.appendChild(document.createTextNode(dataArray[i][prop]));

                objRow.appendChild(objCol);
            }
        }

        objTable.appendChild(objRow);
    }

    objResultContents.appendChild(objTable);
}

function loadDataView(db) {
    db.transaction(function(t) {
        t.executeSql("SELECT * FROM " + DB_TABLE_NAME, [],
            function(t, r) {
                var resultBuffer = [],
                    i;

                for (i = 0; i < r.rows.length; i++) {
                    resultBuffer.push({
                        id: r.rows.item(i).id || 0,
                        item: r.rows.item(i).item || "",
                        price: r.rows.item(i).price || 0,
                        insertDay: r.rows.item(i).insertDay || ""
                    });
                }

                showDataView(resultBuffer);

                return t;
            },
            function(t, e) {
                alert("Error dataview:" + e.message);

                return t;
            });
    });
}

function submitNewRecord() {
    var data = {
            item: "NoName",
            price: 0
        },
        txtItem = document.querySelector("#txtItem"),
        txtPrice = document.querySelector("#txtPrice");

    if (txtItem.value) {
        data.item = txtItem.value;
    }
    if (txtPrice.value) {
        data.price = txtPrice.value;
    }

    insertData(db, data);

    onSuccess({
        message: "Item : " + data.item + " / Price : " + data.price
    });
    alert("Data Inserted");
    txtItem.value = "";
    txtPrice.value = "";
}

function changePage(destPage) {
    var i,
        objPage;

    for (i = 0; i < pageList.length; i++) {
        objPage = document.querySelector("#" + pageList[i]);
        objPage.style.display = "none";
    }

    document.querySelector("#" + destPage).style.display = "block";
    pageNow = destPage;
}

function keyEventCB(event) {
    if (event.keyName === 'back') {
        if (pageNow === "pageMain") {
            try {
                tizen.application.getCurrentApplication().exit();
            } catch (ignore) {

            }
        } else {
            changePage("pageMain");
        }
    }
}

function setDefaultSize() {
    var divResult = document.querySelector("#resultDetail");

    divResult.style.height = (document.height - 50) + "px";
}

function setDefaultEvents() {
    var btnSubmit = document.querySelector("#btnSubmit"),
        btnClear = document.querySelector("#btnClear"),
        btnInputPage = document.querySelector("#btnInputPage"),
        btnResultPage = document.querySelector("#btnResultPage"),
        btnInputBack = document.querySelector("#btnInputBack"),
        btnResultBack = document.querySelector("#btnResultBack");

    document.addEventListener("tizenhwkey", keyEventCB);

    btnResultPage.addEventListener("click", function() {
        loadDataView(db);
        changePage("pageResult");
    });
    btnInputPage.addEventListener("click", function() {
        changePage("pageInput");
    });

    btnInputBack.addEventListener("click", function() {
        changePage("pageMain");
    });
    btnSubmit.addEventListener("click", submitNewRecord);

    btnClear.addEventListener("click", function() {
        clearDataView(db);
    });
    btnResultBack.addEventListener("click", function() {
        changePage("pageMain");
    });

    loadDataView(db);
}

//Initialize function
var init = function() {
    openDB();
    createTable(db);
    setDefaultSize();
    setDefaultEvents();
};

window.onload = init;
